/*
 * YVisualizationManager.java
 *
 * Created on 17. April 2000, 21:15
 */

package YARB.Main;

import YARB.*;
import YARB.FileSources.*;
import YARB.Filter.*;
import YARB.Viewer.*;
/*import YARB.Interfaces.IConfigurable;
import YARB.Interfaces.IVisualContainer;
import YARB.Interfaces.IFileSource;
import YARB.Interfaces.IDataFilter;
import YARB.Interfaces.IDataViewer;
*/
import YARB.Interfaces.*;

import java.util.ArrayList;
import java.lang.*;


/**
 * This is the main dataholder for visualization (but not pattern). uses the 'singleton' idiom
 * @author  Thomas
 * @version
 */
public class YVisualizationManager extends Object
{
  /** The one and only YVisualizationManager object */
  private static YVisualizationManager m_Exemplar=null;
  /** Counter for created containers
   *  used for assigning unique names 
   */
  private static int  m_nContainerCount;
  /** Counter for created onbects 
   * used for assigning unique names 
   */
  private static int  m_nObjectCount;

  /**
   * Private constructor prevents form being created outside.
   */
  private YVisualizationManager()
  {
        m_VC= new ArrayList();
        
        m_nContainerCount=0;
        m_nObjectCount=0;
        

  }

  /**
   * This function returns the one and only object.
   *  The object is created on demand.
   */
  public static YVisualizationManager exemplar()
  {
    if(m_Exemplar==null)
    {
      m_Exemplar= new YVisualizationManager();
    };

    return m_Exemplar;
  }



  //* Variable deklarations:
  //

  private ArrayList   m_VC;
  private String[][]  m_AvailableContainer =    { {"YARB.Main.YDefaultVisualContainer","Default","Default container"}
                                                };
  private String[][]  m_AvailableFileSources =  { {"YARB.FileSources.YFSDummy","Dummy","Dummy file source (for test only)"},
                                                  {"YARB.FileSources.YFSSNNS","SNNS","File source for SNNS files (pattern and result files)"}
                                                };
  private String[][]  m_AvailableDataFilter =   { {"YARB.Filter.YDFOne2One","1:1","1:1 data filter. Does nothing."},
                                                  {"YARB.Filter.YDFWTA", "WTA","The Winner takes All."},
                                                  {"YARB.Filter.YDFArithmetics","Arith","Does some arithmetic operations (+,-,*,/,^n)"}
                                                };

  private String[][]  m_AvailableDataViewer =   { {"YARB.Viewer.YDVPlainText","Plain text","Plain text viewer."},
                                                  {"YARB.Viewer.0YDVTable","Table","Shows the data in tabular format."}
                                                };
                                                
  
  
  public int getNumbVContainers(){
    return m_VC.size();
  }

  public IVisualContainer getVContainer(int nPos){
    return (IVisualContainer) m_VC.get(nPos);
  }


  /**
  * Creates a class from the name (by reflection) and checks whether the given interface is implemented or not.
  * When the function fails, it returns null.
  */
  private Object createClassFromNameAndCheckInterface(String className, String interfaceName)
  {
    try {
            // load the class and search for the required interface
            Class classDefinition = Class.forName(className);
            Class[] theInterfaces = classDefinition.getInterfaces();
            for (int i = 0; i < theInterfaces.length; i++) 
            {
              String str=theInterfaces[i].getName();
               // interface found? then create a new instance, add the container to our list and return it.
               if( str.equals(interfaceName))
               {
                  Object o= classDefinition.newInstance();
                 return o;                  
                };
            }          
          
            // oops. interface not found!
            System.out.println("Error: Loaded class " + className + " didn't implement "  + interfaceName);
          } 
          catch (InstantiationException e) {
            System.out.println(e);    
          } 
          catch (IllegalAccessException e) {
            System.out.println(e);
          } 
          catch (ClassNotFoundException e) {
            System.out.println(e);
          }
          
      return null;
  }

  private String searchShortNameFromClassName(String className, String [][] a)
  {
    String str="";
    
    for (int i=0; i<a.length; ++i) 
    {
      if(a[i].length>=2) {
        if(className.equals(a[i][0]))  str=a[i][1];
      };
    };
    
    return str;
  }
  
   /**
  * Get all known containers
  */
  public String [][] getAvailaibleContainers(){
    return m_AvailableContainer;
  }
  
  /**
  * create a new Container from a class name (by reflection)
  */
  public IVisualContainer createContainer(String className)
  {   
      if(className==null)
      {
        className=m_AvailableContainer[0][0];
      };
      
      IVisualContainer vc = (IVisualContainer) createClassFromNameAndCheckInterface(className, "YARB.Interfaces.IVisualContainer");
     
      if(vc != null)
      {
        m_VC.add(vc);
        vc.setName("Config " + ++m_nContainerCount);
      }
      
      return vc;                  
  }

  /**
  * Delete the container with the spezified index
  */
  public void deleteContainer(int nPos)
  {
      m_VC.remove(nPos);
  }
  

  
  /**
  * Get all known file sources
  */
  public String [][] getAvailableFileSources(){
    return m_AvailableFileSources;
  }
  
  
/**
  * create a new file source from a class name (by reflection)
  */
  public IFileSource createFileSource(String className)
  {   
      if(className==null)
      {
        className=m_AvailableFileSources[0][0];
      };
      
      IFileSource fs= (IFileSource) createClassFromNameAndCheckInterface(className, "YARB.Interfaces.IFileSource");
      if(fs != null)
      {
        fs.setName("File " + ++m_nObjectCount + " (" + searchShortNameFromClassName(className,m_AvailableFileSources) + ")");
      };
      return fs;
  }
  
  
  /**
  * Get all known data filters
  */
  public String [][] getAvailableDataFilter(){
    return m_AvailableDataFilter;
  }
  
  /**
  * create a new data filter from a class name (by reflection)
  */
  public IDataFilter createDataFilter(String className)
  {   
      if(className==null)
      {
        className=m_AvailableDataFilter[0][0];
      };
      
      IDataFilter df = (IDataFilter) createClassFromNameAndCheckInterface(className, "YARB.Interfaces.IDataFilter");
      if(df != null)
      {
        df.setName("Filter "  + ++m_nObjectCount + " (" + searchShortNameFromClassName(className,m_AvailableDataFilter) + ")");
      };
      return df;
      
  }

   
  /**
  * Get all known data viewers 
  */  
  public String [][] getAvailableDataViewer(){
    return m_AvailableDataViewer;
  }

  /**
  * create a new data viewer from a class name (by reflection)
  */
  public IDataViewer createDataViewer(String className)
  {   
      if(className==null)
      {
        className=m_AvailableDataViewer[0][0];
      };
      
      IDataViewer dv = (IDataViewer) createClassFromNameAndCheckInterface(className, "YARB.Interfaces.IDataViewer");
      if(dv != null)
      {
        dv.setName("Viewer " + ++m_nObjectCount + " (" + searchShortNameFromClassName(className,m_AvailableDataViewer) + ")" );
      };
      return dv;
  }


  
  /**
  * Returns the container assiciated to the given object
  */
  public IVisualContainer getContainerByObject(IConfigurable o)
  {

    IVisualContainer cont;
    
    for(int i=0; i<m_VC.size();i++)
    {
      cont=(IVisualContainer)m_VC.get(i);
      
      if(cont==o || cont.getFileSource()==o ||cont.getDataView()==o){
        return cont;
      };
      
      for(int k=0;k<cont.numbOfDataFilter();k++)
      {
        if(cont.getDataFilter(k)==o){
          return cont;
        };
      };
    };
    
    return null;
  }

}
